@@ -6,7 +6,7 @@ from tqdm import tqdm
 import xarray as xr
 import os
 from pathlib import Path
-from typing import Union
+from typing import Union, Optional, Any
 import tarfile
 from urllib.request import urlopen
 import pandas as pd
@@ -14,56 +14,71 @@ import numpy as np
 from tzfpy import get_tz
 from hydroutils import hydro_file
 from hydrodataset import CACHE_DIR, HydroDataset
+from hydrodataset.hydro_dataset import StandardVariable
 
 
 class Caravan(HydroDataset):
-    def __init__(self, data_path, download=False, region="Global", cache_path=None):
+    def __init__(self, data_path: str, region: Optional[str] = None) -> None:
         """
         Initialization for Caravan dataset
 
+        TODO: The newest version is Version 1.6, but now we only support Version 0.3
+
         Parameters
         ----------
         data_path
             where we put the dataset
-        download
-            if true, download
         region
             the region can be US, AUS, BR, CL, GB, CE, NA (North America, meaning HYSETS)
-        cache_path
-            the path to cache the dataset
         """
-        super().__init__(data_path, cache_path=cache_path)
-        self.region = region
+        self.data_path = os.path.join(data_path, "CARAVAN")
+        super().__init__(self.data_path)
+        self.region = "Global" if region is None else region
         region_name_dict = self.region_name_dict
-        if region == "Global":
+        if self.region == "Global":
             self.region_data_name = list(region_name_dict.values())
         else:
-            self.region_data_name = region_name_dict[region]
+            self.region_data_name = region_name_dict[self.region]
         self.data_source_description = self.set_data_source_describe()
-        if download:
-            self.download_data_source()
         try:
             self.is_data_ready()
         except FileNotFoundError as e:
             warnings.warn(e)
-            print(
-                "Please download and unzip the dataset first: just set download=True if you have manually downloaded zip files when you first initialize caravan."
-            )
+            print("Downloading dataset...")
+            self.download_data_source()
+            print("Dataset downloaded successfully.")
+            print("Checking if dataset is ready...")
+            self.is_data_ready()
+            print("Dataset is ready.")
         self.sites = self.read_site_info()
 
     @property
     def _attributes_cache_filename(self):
-        region_name = self.region_data_name
-        if isinstance(region_name, list):
-            region_name = "_".join(region_name)
-        return f"caravan_{region_name}_attributes.nc"
+        return "caravan_attributes.nc"
 
     @property
-    def _attributes_cache_filename(self):
+    def _timeseries_cache_filename(self):
+        """
+        Get glob pattern to match batch cache files.
+
+        Returns
+        -------
+        str
+            Glob pattern to match all batch cache files for the region(s)
+        """
         region_name = self.region_data_name
         if isinstance(region_name, list):
-            region_name = "_".join(region_name)
-        return f"caravan_{region_name}_attributes.nc"
+            # Match all regions: caravan_*_timeseries_batch_*.nc
+            return str(self.cache_dir.joinpath("caravan_*_timeseries_batch_*.nc"))
+        else:
+            # Match specific region: caravan_{region}_timeseries_batch_*.nc
+            return str(
+                self.cache_dir.joinpath(f"caravan_{region_name}_timeseries_batch_*.nc")
+            )
+
+    @property
+    def default_t_range(self):
+        return ["1981-01-01", "2020-12-31"]
 
     @property
     def region_name_dict(self):
@@ -77,6 +92,67 @@ class Caravan(HydroDataset):
             "CE": "lamah",
         }
 
+    _subclass_static_definitions = {
+        "area": {"specific_name": "area", "unit": "km^2"},
+        "p_mean": {"specific_name": "p_mean", "unit": "mm/day"},
+    }
+    _dynamic_variable_mapping = {
+        StandardVariable.STREAMFLOW: {
+            "default_source": "each_region",
+            "sources": {
+                "each_region": {"specific_name": "streamflow", "unit": "mm/day"}
+            },
+        },
+        StandardVariable.PRECIPITATION: {
+            "default_source": "each_region",
+            "sources": {
+                "each_region": {
+                    "specific_name": "total_precipitation_sum",
+                    "unit": "mm/day",
+                },
+            },
+        },
+        StandardVariable.TEMPERATURE_MAX: {
+            "default_source": "each_region",
+            "sources": {
+                "each_region": {"specific_name": "temperature_2m_max", "unit": "°C"},
+            },
+        },
+        StandardVariable.TEMPERATURE_MIN: {
+            "default_source": "each_region",
+            "sources": {
+                "each_region": {"specific_name": "temperature_2m_min", "unit": "°C"},
+            },
+        },
+        StandardVariable.SOLAR_RADIATION: {
+            "default_source": "each_region",
+            "sources": {
+                "each_region": {
+                    "specific_name": "surface_net_solar_radiation_mean",
+                    "unit": "W/m^2",
+                },
+            },
+        },
+        StandardVariable.SNOW_WATER_EQUIVALENT: {
+            "default_source": "each_region",
+            "sources": {
+                "each_region": {
+                    "specific_name": "snow_depth_water_equivalent_mean",
+                    "unit": "mm/day",
+                },
+            },
+        },
+        StandardVariable.POTENTIAL_EVAPOTRANSPIRATION: {
+            "default_source": "each_region",
+            "sources": {
+                "each_region": {
+                    "specific_name": "potential_evaporation_sum",
+                    "unit": "mm/day",
+                }
+            },
+        },
+    }
+
     def get_name(self):
         return "Caravan_" + self.region
 
@@ -93,8 +169,10 @@ class Caravan(HydroDataset):
 
         # We use A_basins_total_upstrm
         # shp file of basins
-        # TODO: Caravan Global is not fully tested yet
-        shp_dir = os.path.join(dataset_dir, "shapefiles", self.region_data_name)
+        if self.region == "Global":
+            shp_dir = os.path.join(dataset_dir, "shapefiles")
+        else:
+            shp_dir = os.path.join(dataset_dir, "shapefiles", self.region_data_name)
         # read the shp in this dir
         shp_files = [f for f in os.listdir(shp_dir) if f.endswith(".shp")]
 
@@ -108,6 +186,7 @@ class Caravan(HydroDataset):
         forcing_dir = flow_dir
         attr_dir = os.path.join(dataset_dir, "attributes")
         ts_csv_dir = os.path.join(dataset_dir, "timeseries", "csv")
+        # TODO: The newest version is Version 1.6, but the url is not updated yet. it's still Version 0.3
         download_url = "https://zenodo.org/record/7944025/files/Caravan.zip"
         return collections.OrderedDict(
             DATASET_DIR=dataset_dir,
@@ -290,7 +369,7 @@ class Caravan(HydroDataset):
         """
         return np.array(["streamflow"])
 
-    def read_object_ids(self, **kwargs) -> np.array:
+    def read_object_ids(self, **kwargs: Any) -> np.array:
         """
         read station ids
 
@@ -311,7 +390,7 @@ class Caravan(HydroDataset):
         gage_id_lst: Union[list, np.array] = None,
         t_range: list = None,
         target_cols: Union[list, np.array] = None,
-        **kwargs,
+        **kwargs: Any,
     ) -> np.array:
         if self.region == "Global":
             return self._read_timeseries_data_global(
@@ -347,11 +426,11 @@ class Caravan(HydroDataset):
 
     def read_relevant_cols(
         self,
-        gage_id_lst: list = None,
-        t_range: list = None,
-        var_lst: list = None,
-        forcing_type="era5land",
-        **kwargs,
+        gage_id_lst: Optional[list] = None,
+        t_range: Optional[list] = None,
+        var_lst: Optional[list] = None,
+        forcing_type: str = "era5land",
+        **kwargs: Any,
     ) -> np.ndarray:
         """_summary_
 
@@ -403,8 +482,12 @@ class Caravan(HydroDataset):
         return data[var_lst]
 
     def read_constant_cols(
-        self, gage_id_lst=None, var_lst=None, is_return_dict=False, **kwargs
-    ):
+        self,
+        gage_id_lst: Optional[list] = None,
+        var_lst: Optional[list] = None,
+        is_return_dict: bool = False,
+        **kwargs: Any,
+    ) -> Union[dict, np.ndarray]:
         """
         Read Attributes data
 
@@ -501,10 +584,10 @@ class Caravan(HydroDataset):
         data = pd.concat(attr)
         return data.to_dict("index") if is_return_dict else data.values
 
-    def read_mean_prep(self, object_ids) -> np.array:
+    def read_mean_prep(self, object_ids: Optional[list] = None) -> np.array:
         return self.read_constant_cols(object_ids, ["p_mean"], is_return_dict=False)
 
-    def cache_attributes_xrdataset(self):
+    def cache_attributes_xrdataset(self) -> None:
         """cache attributes in xr dataset"""
         import pint_xarray  # noqa
         import pint
@@ -677,7 +760,7 @@ class Caravan(HydroDataset):
         cache_attr_file = self.cache_dir.joinpath(self._attributes_cache_filename)
         ds.to_netcdf(cache_attr_file)
 
-    def cache_xrdataset(self, **kwargs):
+    def cache_xrdataset(self, **kwargs: Any) -> None:
         """
         Save all attr data in a netcdf file in the cache directory,
         ts data are already nc format
@@ -692,12 +775,20 @@ class Caravan(HydroDataset):
         self.cache_attributes_xrdataset()
         self.cache_timeseries_xrdataset(checkregion)
 
-    def read_timeseries(self, basin_ids=None, t_range_list: list = None, var_lst=None):
+    def read_timeseries(
+        self,
+        region: str,
+        basin_ids: Optional[list] = None,
+        t_range_list: Optional[list] = None,
+        var_lst: Optional[list] = None,
+    ) -> np.ndarray:
         """
         Read time-series data from csv files
 
         Parameters
         ----------
+        region
+            the region can be US, AUS, BR, CL, GB, CE, NA (North America, meaning HYSETS)
         basin_ids
             station ids
         t_range
@@ -715,9 +806,8 @@ class Caravan(HydroDataset):
         if var_lst is None:
             var_lst = self.get_relevant_cols()
         if t_range_list is None:
-            t_range_list = ["1980-01-01", "2023-12-31"]
+            t_range_list = self.default_t_range
         ts_dir = self.data_source_description["TS_CSV_DIR"]
-        region = self.region_data_name
         t_range = pd.date_range(start=t_range_list[0], end=t_range_list[-1], freq="1D")
         nt = len(t_range)
         x = np.full([len(basin_ids), nt, len(var_lst)], np.nan)
@@ -735,7 +825,7 @@ class Caravan(HydroDataset):
                 x[k, ind2, j] = ts_data[var_lst[j]][ind1].values
         return x
 
-    def _get_unit_json(self, onebasinid, region_name):
+    def _get_unit_json(self, onebasinid: str, region_name: str) -> dict:
         ancfile4unit = os.path.join(
             self.data_source_description["FLOW_DIR"],
             region_name,
@@ -772,7 +862,9 @@ class Caravan(HydroDataset):
         result_dict["streamflow"] = "mm/day"
         return result_dict
 
-    def cache_timeseries_xrdataset(self, checkregion, **kwargs):
+    def cache_timeseries_xrdataset(
+        self, checkregion: Optional[str] = None, **kwargs: Any
+    ) -> None:
         if checkregion is not None:
             regions = self.region_data_name if checkregion == "all" else [checkregion]
             self._check_data(regions)
@@ -793,7 +885,7 @@ class Caravan(HydroDataset):
             batchsize = kwargs.get("batchsize", 100)
             t_range = kwargs.get("t_range", None)
             if t_range is None:
-                t_range = ["1980-01-01", "2023-12-31"]
+                t_range = self.default_t_range
             times = (
                 pd.date_range(start=t_range[0], end=t_range[-1], freq="1D")
                 .strftime("%Y-%m-%d %H:%M:%S")
@@ -810,8 +902,10 @@ class Caravan(HydroDataset):
                 # we make sure the basin ids are sorted
                 assert all(x <= y for x, y in zip(basin_batch, basin_batch[1:]))
                 data = self.read_timeseries(
+                    region=region,
                     basin_ids=basin_batch,
                     t_range_list=t_range,
+                    var_lst=variables,
                 )
                 dataset = xr.Dataset(
                     data_vars={
@@ -903,15 +997,56 @@ class Caravan(HydroDataset):
                     )
                     the_ncfile_data.to_netcdf(the_ncfile)
 
-    def read_attr_xrdataset(self, gage_id_lst=None, var_lst=None, **kwargs):
-        # Define the path to the attributes file
+    def read_attr_xrdataset(
+        self,
+        gage_id_lst: Optional[list] = None,
+        var_lst: Optional[list] = None,
+        to_numeric: bool = True,
+        **kwargs: Any,
+    ) -> xr.Dataset:
+        """
+        Read attribute data as xarray Dataset.
+
+        This method supports both standardized variable names and dataset-specific names.
+        If var_lst contains standardized names (e.g., 'area', 'p_mean'),
+        it will use the parent class implementation. Otherwise, it reads the raw attributes.
+
+        Args:
+            gage_id_lst (list, optional): List of gauge IDs to select.
+            var_lst (list, optional): List of variable names (standardized or raw).
+            to_numeric (bool, optional): Whether to convert non-numeric variables to codes.
+            **kwargs: Additional keyword arguments.
+
+        Returns:
+            xr.Dataset: Dataset containing the requested attributes.
+        """
+        # Check if var_lst contains only standardized variable names
+        if var_lst is not None:
+            all_standard = all(
+                var in self._static_variable_definitions for var in var_lst
+            )
+            if all_standard:
+                # Use parent class implementation for standardized variables
+                return super().read_attr_xrdataset(
+                    gage_id_lst=gage_id_lst,
+                    var_lst=var_lst,
+                    to_numeric=to_numeric,
+                    **kwargs,
+                )
+
+        # For raw attribute names or mixed names, use direct reading
         file_path = self.cache_dir.joinpath(self._attributes_cache_filename)
 
+        if not os.path.isfile(file_path):
+            self.cache_attributes_xrdataset()
+
         # Open the dataset
         ds = xr.open_dataset(file_path)
 
         # Select the basins
-        ds = ds.sel(basin=gage_id_lst)
+        if gage_id_lst is not None:
+            gage_id_lst = [str(gid) for gid in gage_id_lst]
+            ds = ds.sel(basin=gage_id_lst)
 
         # If relevant columns (attributes) are specified, select them
         if var_lst:
@@ -919,15 +1054,22 @@ class Caravan(HydroDataset):
 
         return ds
 
-    def read_ts_xrdataset(self, gage_id_lst, t_range, var_lst, **kwargs):
-        region_name = self.region_data_name
-        if isinstance(region_name, list):
-            region_name = "_".join(region_name)
-        file_paths = sorted(
-            glob.glob(
-                os.path.join(self.cache_dir, f"*caravan_{region_name}_*timeseries*.nc")
-            )
-        )
+    def _load_ts_dataset(self, **kwargs: Any) -> xr.Dataset:
+        """Load the time series dataset from cache files.
+
+        This method overrides the parent method to handle multiple batch files.
+
+        Args:
+            **kwargs: Additional keyword arguments.
+                parallel (bool, optional): Whether to use parallel reading. Defaults to False.
+
+        Returns:
+            xr.Dataset: The loaded time series dataset.
+        """
+        file_paths = sorted(glob.glob(self._timeseries_cache_filename))
+        if len(file_paths) == 0:
+            self.cache_timeseries_xrdataset(checkregion="all")
+            file_paths = sorted(glob.glob(self._timeseries_cache_filename))
 
         # Open the dataset in a lazy manner using dask
         parallel = kwargs.get("parallel", False)
@@ -937,34 +1079,64 @@ class Caravan(HydroDataset):
             concat_dim="basin",
             parallel=parallel,
         )
-        # If relevant columns are specified, select them
-        if var_lst:
-            combined_ds = combined_ds[var_lst]
-        if t_range:
-            combined_ds = combined_ds.sel(time=slice(*t_range))
-        if gage_id_lst:
-            combined_ds = combined_ds.sel(basin=gage_id_lst)
-
-        # some units are not recognized by pint_xarray, hence we manually set them
-        unit_mapping = {"W/m2": "watt / meter ** 2", "m3/m3": "meter^3/meter^3"}
-
-        for var in combined_ds.data_vars:
-            if "units" not in combined_ds[var].attrs:
-                unit = _extract_unit(var, combined_ds.attrs["Units"])
-                # If the extracted unit is in the mapping dictionary, replace it
-                unit = unit_mapping.get(unit, unit)
-                combined_ds[var].attrs["units"] = unit
-
         return combined_ds
 
+    def read_ts_xrdataset(
+        self,
+        gage_id_lst: Optional[list] = None,
+        t_range: Optional[list] = None,
+        var_lst: Optional[list] = None,
+        sources: Optional[dict] = None,
+        **kwargs: Any,
+    ) -> xr.Dataset:
+        """
+        Reads time series data as an xarray Dataset with standardized variable names.
+
+        Args:
+            gage_id_lst (list, optional): List of gauge IDs to select.
+            t_range (list, optional): Time range as [start, end].
+            var_lst (list, optional): List of standard variable names (e.g., StandardVariable.STREAMFLOW).
+            sources (dict, optional): Dictionary mapping standard variable names to their sources.
+            **kwargs: Additional keyword arguments. Notably:
+                parallel (bool): Whether to use parallel reading (default: False).
+
+        Returns:
+            xr.Dataset: Dataset with standardized variable names.
+        """
+        # Use parent class implementation for variable mapping logic
+        final_ds = super().read_ts_xrdataset(
+            gage_id_lst=gage_id_lst,
+            t_range=t_range,
+            var_lst=var_lst,
+            sources=sources,
+            **kwargs,
+        )
+
+        # Fix units that are not recognized by pint_xarray
+        unit_mapping = {
+            "W/m2": "watt / meter ** 2",
+            "m3/m3": "meter^3/meter^3",
+            "W/m^2": "watt / meter ** 2",
+        }
+
+        for var in final_ds.data_vars:
+            if "units" in final_ds[var].attrs:
+                unit = final_ds[var].attrs["units"]
+                # If the unit is in the mapping dictionary, replace it
+                final_ds[var].attrs["units"] = unit_mapping.get(unit, unit)
+
+        return final_ds
+
     @property
     def streamflow_unit(self):
         return "mm/d"
 
-    def read_area(self, gage_id_lst=None):
+    def read_area(self, gage_id_lst: Optional[list] = None) -> xr.Dataset:
         return self.read_attr_xrdataset(gage_id_lst, ["area"])
 
-    def read_mean_prcp(self, gage_id_lst=None, unit="mm/d"):
+    def read_mean_prcp(
+        self, gage_id_lst: Optional[list] = None, unit: str = "mm/d"
+    ) -> xr.Dataset:
         return self.read_attr_xrdataset(gage_id_lst, ["p_mean"])
 
 
