@@ -1,7 +1,7 @@
 """
 Author: Wenyu Ouyang
 Date: 2022-09-05 23:20:24
-LastEditTime: 2025-10-30 09:11:40
+LastEditTime: 2025-11-06 19:26:20
 LastEditors: Wenyu Ouyang
 Description: main modules for hydrodataset
 FilePath: \hydrodataset\hydrodataset\hydro_dataset.py
@@ -27,6 +27,9 @@ class StandardVariable:
 
     PRECIPITATION = "precipitation"
     CRAINF_FRAC = "crainf_frac"  # Fraction of total precipitation that is convective
+    PRECIPITATION_MIN = "precipitation_min"
+    PRECIPITATION_MAX = "precipitation_max"
+    PRECIPITATION_MEDIAN = "precipitation_median"
 
     TEMPERATURE_MAX = "temperature_max"
     TEMPERATURE_MIN = "temperature_min"
@@ -36,26 +39,72 @@ class StandardVariable:
     RELATIVE_DAYLIGHT_DURATION = "relative_daylight_duration"
 
     SOLAR_RADIATION = "solar_radiation"
+    SOLAR_RADIATION_MIN = "solar_radiation_min"
+    SOLAR_RADIATION_MAX = "solar_radiation_max"
+    SOLAR_RADIATION_MEDIAN = "solar_radiation_median"
+    THERMAL_RADIATION = "thermal_radiation"
+    THERMAL_RADIATION_MIN = "thermal_radiation_min"
+    THERMAL_RADIATION_MAX = "thermal_radiation_max"
     LONGWAVE_SOLAR_RADIATION = "longwave_solar_radiation"
 
     SNOW_WATER_EQUIVALENT = "snow_water_equivalent"
+    SNOW_WATER_EQUIVALENT_MIN = "snow_water_equivalent_min"
+    SNOW_WATER_EQUIVALENT_MAX = "snow_water_equivalent_max"
+    SNOW_DEPTH = "snow_depth"
+    SNOW_COVER = "snow_cover"
+    SNOW_SUBLIMATION = "snow_sublimation"
+    SNOW_DENSITY = "snow_density"
 
     VAPOR_PRESSURE = "vapor_pressure"
 
     SURFACE_PRESSURE = "surface_pressure"
+    SURFACE_PRESSURE_MIN = "surface_pressure_min"
+    SURFACE_PRESSURE_MAX = "surface_pressure_max"
 
     WIND_SPEED = "wind_speed"
-    MERIDIONAL_WIND_SPEED = "meridional_wind_speed"
+    U_WIND_SPEED = "u_wind_speed"
+    U_WIND_SPEED_MIN = "u_wind_speed_min"
+    U_WIND_SPEED_MAX = "u_wind_speed_max"
+    V_WIND_SPEED = "v_wind_speed"
+    V_WIND_SPEED_MIN = "v_wind_speed_min"
+    V_WIND_SPEED_MAX = "v_wind_speed_max"
+    WIND_DIR = "wind_dir"
+    LOW_LEVEL_WIND_SHEAR = "low_level_wind_shear"
+    DEEP_LEVEL_WIND_SHEAR = "deep_level_wind_shear"
 
     RELATIVE_HUMIDITY = "relative_humidity"
+    SPECIFIC_HUMIDITY = "specific_humidity"
     RELATIVE_HUMIDITY_MIN = "relative_humidity_min"
+    RELATIVE_HUMIDITY_MAX = "relative_humidity_max"
+    RELATIVE_HUMIDITY_MEDIAN = "relative_humidity_median"
+    TOTAL_COLUMN_WATER_VAPOUR = "total_column_water_vapour"
 
     CAPE = "cape"  # Convective available potential energy
+    CIN = "cin"  # Convective inhibition
 
     POTENTIAL_EVAPOTRANSPIRATION = "potential_evapotranspiration"
     EVAPORATION = "evaporation"
     EVAPOTRANSPIRATION = "evapotranspiration"
 
+    SOIL_MOISTURE = "soil_moisture"
+    VOLUMETRIC_SOIL_WATER_LAYER1 = "volumetric_soil_water_layer1"  # 0-7cm
+    VOLUMETRIC_SOIL_WATER_LAYER1_MIN = "volumetric_soil_water_layer1_min"
+    VOLUMETRIC_SOIL_WATER_LAYER1_MAX = "volumetric_soil_water_layer1_max"
+    VOLUMETRIC_SOIL_WATER_LAYER2 = "volumetric_soil_water_layer2"  # 7-28cm
+    VOLUMETRIC_SOIL_WATER_LAYER2_MIN = "volumetric_soil_water_layer2_min"
+    VOLUMETRIC_SOIL_WATER_LAYER2_MAX = "volumetric_soil_water_layer2_max"
+    VOLUMETRIC_SOIL_WATER_LAYER3 = "volumetric_soil_water_layer3"  # 28-100cm
+    VOLUMETRIC_SOIL_WATER_LAYER3_MIN = "volumetric_soil_water_layer3_min"
+    VOLUMETRIC_SOIL_WATER_LAYER3_MAX = "volumetric_soil_water_layer3_max"
+    VOLUMETRIC_SOIL_WATER_LAYER4 = "volumetric_soil_water_layer4"  # 100-289cm
+    VOLUMETRIC_SOIL_WATER_LAYER4_MIN = "volumetric_soil_water_layer4_min"
+    VOLUMETRIC_SOIL_WATER_LAYER4_MAX = "volumetric_soil_water_layer4_max"
+
+    MIN_RAIN_RATE = "min_rain_rate"
+    MAX_RAIN_RATE = "max_rain_rate"
+
+    GROUND_HEAT_FLUX = "ground_heat_flux"
+
 
 class HydroDataset(ABC):
     """An interface for Hydrological Dataset
@@ -352,7 +401,7 @@ class HydroDataset(ABC):
         Args:
             gage_id_lst: A list of basin identifiers.
             var_lst: A list of **standard** attribute names to retrieve.
-                If None, all available static features will be returned.
+                If None, nothing will be returned.
             to_numeric: If True, converts all non-numeric variables to numeric codes
                 and stores the original labels in the variable's attributes.
                 Defaults to True.
@@ -361,8 +410,8 @@ class HydroDataset(ABC):
             An xarray Dataset containing the attribute data for the requested basins,
             with variables named using the standard names.
         """
-        if var_lst is None:
-            var_lst = self.get_available_static_features()
+        if not var_lst:
+            return None
 
         # 1. Translate standard names to dataset-specific names
         target_vars_to_fetch = []
@@ -419,6 +468,26 @@ class HydroDataset(ABC):
                 converted_ds[var_name] = new_da
         return converted_ds
 
+    def _load_ts_dataset(self, **kwargs):
+        """
+        Loads the time series dataset from cache.
+
+        This method can be overridden by subclasses to implement different loading
+        strategies (e.g., loading multiple files).
+
+        Args:
+            **kwargs: Additional keyword arguments for loading.
+
+        Returns:
+            xarray.Dataset: The loaded time series dataset.
+        """
+        ts_cache_file = self.cache_dir.joinpath(self._timeseries_cache_filename)
+
+        if not os.path.isfile(ts_cache_file):
+            self.cache_timeseries_xrdataset()
+
+        return xr.open_dataset(ts_cache_file)
+
     def read_ts_xrdataset(
         self,
         gage_id_lst: list = None,
@@ -478,12 +547,7 @@ class HydroDataset(ABC):
                 rename_map[actual_var_name] = output_name
 
         # Read data from cache using actual variable names
-        ts_cache_file = self.cache_dir.joinpath(self._timeseries_cache_filename)
-
-        if not os.path.isfile(ts_cache_file):
-            self.cache_timeseries_xrdataset()
-
-        ts = xr.open_dataset(ts_cache_file)
+        ts = self._load_ts_dataset(**kwargs)
         missing_vars = [v for v in target_vars_to_fetch if v not in ts.data_vars]
         if missing_vars:
             # To provide a better error message, map back to standard names
